@@ -948,11 +948,6 @@ move_here (FrArchive *archive)
 	if (content_uri == NULL)
 		return;	
 
-	if (! path_is_dir (content_uri)) {
-		g_free (content_uri);
-		return;
-	}
-
 	parent = remove_level_from_path (content_uri);
 		
 	if (uricmp (parent, archive->priv->extraction_destination) == 0) {
@@ -1281,9 +1276,9 @@ fr_archive_load__error (FrArchive  *archive,
 
 
 static void
-local_local_archive (FrArchive  *archive,
-		     const char *uri,
-		     const char *password)
+load_local_archive (FrArchive  *archive,
+		    const char *uri,
+		    const char *password)
 {
 	FrCommand  *tmp_command;
 	const char *mime_type = NULL;
@@ -1360,7 +1355,7 @@ copy_remote_file__step2 (FrArchive      *archive,
 	if (result != GNOME_VFS_OK) 
 		fr_archive_load__error (archive, gnome_vfs_result_to_string (result));
 	else
-		local_local_archive (archive, uri, password);
+		load_local_archive (archive, uri, password);
 }
 
 
@@ -1511,7 +1506,7 @@ fr_archive_load_local (FrArchive  *archive,
 		       FR_ACTION_LOADING_ARCHIVE);
 
 	fr_archive_set_uri (archive, uri);
-	local_local_archive (archive, uri, password);
+	load_local_archive (archive, uri, password);
 	
 	return TRUE;	
 }
@@ -1605,12 +1600,12 @@ static FileData *
 find_file_in_archive (FrArchive *archive,
 		      char      *path)
 {
-	GList *scan;
+	int i;
 
 	g_return_val_if_fail (path != NULL, NULL);
 
-	for (scan = archive->command->file_list; scan; scan = scan->next) {
-		FileData *fdata = scan->data;
+	for (i = 0; i < archive->command->files->len; i++) {
+		FileData *fdata = g_ptr_array_index (archive->command->files, i);
 		if (strcmp (path, fdata->original_path) == 0)
 			return fdata;
 	}
@@ -1710,8 +1705,12 @@ convert_to_local_file_list (GList *file_list)
 	GList *scan;
 
 	for (scan = file_list; scan; scan = scan->next) {
-		char *filename = scan->data;
-		local_file_list = g_list_prepend (local_file_list, get_local_path_from_uri (filename));
+		char *uri = scan->data;
+		char *local_filename;
+		
+		local_filename = get_local_path_from_uri (uri);
+		if (local_filename != NULL)
+			local_file_list = g_list_prepend (local_file_list, local_filename);
 	}
 
 	return local_file_list;
@@ -2586,47 +2585,75 @@ file_is_in_subfolder_of (const char *filename,
 }
 
 
+static gboolean
+archive_type_has_issues_deleting_non_empty_folders (FrArchive *archive)
+{
+	/*if ((archive->command->files == NULL) || (archive->command->files->len == 0))
+		return FALSE;  FIXME: test with extract_here */
+		
+	return ((archive->command->file_type == FR_FILE_TYPE_TAR)
+		|| (archive->command->file_type == FR_FILE_TYPE_TAR_BZ)
+		|| (archive->command->file_type == FR_FILE_TYPE_TAR_BZ2)
+		|| (archive->command->file_type == FR_FILE_TYPE_TAR_GZ)
+		|| (archive->command->file_type == FR_FILE_TYPE_TAR_LZOP)
+		|| (archive->command->file_type == FR_FILE_TYPE_TAR_COMPRESS));
+}
+
+
 /* Note: all paths unescaped. */
 static void
 archive_remove (FrArchive *archive,
 		GList     *file_list)
 {
 	gboolean  file_list_created = FALSE;
-	GList    *tmp_file_list;
+	GList    *tmp_file_list = NULL;
+	gboolean  tmp_file_list_created = FALSE;
 	GList    *e_file_list;
 	GList    *scan;
-	GList    *folders_to_remove;
-
-	/* file_list == NULL means delete all files in archive. */
+	
+	/* file_list == NULL means delete all the files in the archive. */
 
 	if (file_list == NULL) {
-		for (scan = archive->command->file_list; scan != NULL; scan = scan->next) {
-			FileData *fdata = (FileData*) scan->data;
+		int i;
 
+		for (i = 0; i < archive->command->files->len; i++) {
+			FileData *fdata = g_ptr_array_index (archive->command->files, i);
 			file_list = g_list_prepend (file_list, fdata->original_path);
 		}
+
 		file_list_created = TRUE;
 	}
 
-	/* remove from the list the files inside folders to be removed. */
+	if (archive_type_has_issues_deleting_non_empty_folders (archive)) {
+		GList *folders_to_remove;
+		
+		/* remove from the list the files contained in folders to be
+		 * removed. */
 
-	folders_to_remove = NULL;
-	for (scan = file_list; scan != NULL; scan = scan->next) {
-		char *path = scan->data;
+		folders_to_remove = NULL;
+		for (scan = file_list; scan != NULL; scan = scan->next) {
+			char *path = scan->data;
 
-		if (path[strlen (path) - 1] == '/')
-			folders_to_remove = g_list_prepend (folders_to_remove, path);
+			if (path[strlen (path) - 1] == '/')
+				folders_to_remove = g_list_prepend (folders_to_remove, path);
+		}
+
+		if (folders_to_remove != NULL) {
+			tmp_file_list = NULL;
+			for (scan = file_list; scan != NULL; scan = scan->next) {
+				char *path = scan->data;
+
+				if (! file_is_in_subfolder_of (path, folders_to_remove))
+					tmp_file_list = g_list_prepend (tmp_file_list, path);
+			}
+			tmp_file_list_created = TRUE;
+			g_list_free (folders_to_remove);
+		}
 	}
 
-	tmp_file_list = NULL;
-	for (scan = file_list; scan != NULL; scan = scan->next) {
-		char *path = scan->data;
-
-		if (! file_is_in_subfolder_of (path, folders_to_remove))
-			tmp_file_list = g_list_prepend (tmp_file_list, path);
-	}
-
-	g_list_free (folders_to_remove);
+	if (! tmp_file_list_created)
+		tmp_file_list = g_list_copy (file_list);
+			
 	if (file_list_created)
 		g_list_free (file_list);
 
@@ -2902,8 +2929,8 @@ compute_list_base_path (const char *base_dir,
 static gboolean
 archive_type_has_issues_extracting_non_empty_folders (FrArchive *archive)
 {
-	if (archive->command->file_list == NULL)
-		return FALSE;
+	/*if ((archive->command->files == NULL) || (archive->command->files->len == 0))
+		return FALSE;  FIXME: test with extract_here */
 		
 	return ((archive->command->file_type == FR_FILE_TYPE_TAR)
 		|| (archive->command->file_type == FR_FILE_TYPE_TAR_BZ)
@@ -2931,6 +2958,35 @@ file_list_contains_files_in_this_dir (GList      *file_list,
 }
 
 
+static GList*
+remove_files_contained_in_this_dir (GList      *file_list,
+				    const char *dirname, 
+				    gboolean   *changed)
+{
+	GList *scan;
+
+	*changed = FALSE;
+	
+	for (scan = file_list; scan; /* empty */) {
+		char *filename = scan->data;
+
+		if (path_in_path (dirname, filename)) {
+			GList *next = scan->next;
+			
+			file_list = g_list_remove_link (file_list, scan);
+			g_list_free (scan);
+			*changed = TRUE;
+			
+			scan = next;
+		}
+		else
+			scan = scan->next;
+	}
+	
+	return file_list;
+}
+
+
 /* Note : All paths unescaped.
  * Note2: Do not escape dest_dir it will escaped in fr_command_extract if
  *        needed. */
@@ -2963,14 +3019,13 @@ fr_archive_extract_to_local (FrArchive  *archive,
 
 	extract_all = (file_list == NULL);
 	if (extract_all && ! archive->command->propCanExtractAll) {
-		GList *scan;
-
+		int i;
+		
 		file_list = NULL;
-		for (scan = archive->command->file_list; scan; scan = scan->next) {
-			FileData *fdata = scan->data;
+		for (i = 0; i < archive->command->files->len; i++) {
+			FileData *fdata = g_ptr_array_index (archive->command->files, i);
 			file_list = g_list_prepend (file_list, g_strdup (fdata->original_path));
 		}
-
 		file_list_created = TRUE;
 	}
 
@@ -2986,20 +3041,15 @@ fr_archive_extract_to_local (FrArchive  *archive,
 
 		if (! extract_all && archive_type_has_issues_extracting_non_empty_folders (archive)) {
 			created_filtered_list = TRUE;
-			filtered = NULL;
-			for (scan = file_list; scan; scan = scan->next) {
-				FileData *fdata;
-				char     *archive_list_filename = scan->data;
-
-				fdata = find_file_in_archive (archive, archive_list_filename);
-
-				if (fdata == NULL)
-					continue;
-
-				if (fdata->dir && file_list_contains_files_in_this_dir (file_list, archive_list_filename))
-					continue;
-
-				filtered = g_list_prepend (filtered, fdata->original_path);
+			filtered = g_list_copy (file_list);
+			for (scan = filtered; scan; /* empty */) {  
+				gboolean changed = FALSE;
+				
+				filtered = remove_files_contained_in_this_dir (filtered, scan->data, &changed);
+				if (changed)
+					scan = filtered;
+				else
+					scan = scan->next;
 			}
 		}
 		else
@@ -3033,11 +3083,11 @@ fr_archive_extract_to_local (FrArchive  *archive,
 				 && ! archive->command->propExtractCanJunkPaths)));
 
 	if (extract_all && ! file_list_created) {
-		GList *scan;
+		int i;
 
 		file_list = NULL;
-		for (scan = archive->command->file_list; scan; scan = scan->next) {
-			FileData *fdata = scan->data;
+		for (i = 0; i < archive->command->files->len; i++) {
+			FileData *fdata = g_ptr_array_index (archive->command->files, i);
 			file_list = g_list_prepend (file_list, g_strdup (fdata->original_path));
 		}
 
